package com.notuvy.singleapp.multiuser;

import org.apache.log4j.Logger;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.net.SocketException;

/**
 * Basic description
 *
 * @author murali
 */
public class BrokerConnection {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger("com.notuvy.singleapp");


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private final String fUser;
    private final Socket fSocket;
    private final ObjectInputStream fInput;
    private final ObjectOutputStream fOutput;


    //------------------------------------------------------------
    //- Constructors

    public BrokerConnection(String pUser, Socket pSocket) throws IOException {
        fUser = pUser;
        fSocket = pSocket;
        fOutput = new ObjectOutputStream(pSocket.getOutputStream());
        fInput = new ObjectInputStream(pSocket.getInputStream());
    }

    /**
     * Create an incoming connection.  This will read the username from the inputstream.
     *
     * @param pSocket incoming socket.
     * @throws IOException
     */
    public BrokerConnection(Socket pSocket) throws IOException {
        fSocket = pSocket;
        fInput = new ObjectInputStream(pSocket.getInputStream());
        fOutput = new ObjectOutputStream(pSocket.getOutputStream());
        fUser = fInput.readUTF();
    }


    //------------------------------------------------------------
    //- Gettors

    public String getUser() {
        return fUser;
    }

    public ObjectInputStream getInput() {
        return fInput;
    }

    public ObjectOutputStream getOutput() {
        return fOutput;
    }


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    protected boolean canCommunicate() {
        // An outgoing socket connection (which is what we have from the broker to the broker client)
        // does not automatically detect that the connection was lost.  (An incoming socket connection
        // immediately sees EOF when the connection is lost).
        // Because of this, we need to perform a simple ping to determine that the connection is still
        // active.  A simple message is sent and a response is received. If this is done without error,
        // then the connection is determined to be alive and well.

        boolean success = false;  // Pessimistic.
        try {
            // LOG.debug("Requesting ping.");
            getOutput().writeInt(MultiUserSingleAppEnforcer.MSG_TYPE_PING);
            getOutput().flush();
            int response = getInput().readInt();
            if (MultiUserSingleAppEnforcer.MSG_TYPE_PING != response) {
                LOG.error("Unexpected ping response: " + response);
            }
            success = true;
        } catch (SocketException se) {
            if (se.getMessage().equals("Broken pipe")) {
                LOG.debug("Existing client connection expired; need to create a new one for current client.");
            } else {
                LOG.error("Ping failed.", se);
            }
        } catch (IOException ioe) {
            LOG.error("Ping failed.", ioe);
        }
        return success;
    }

    protected Object readObjectMessage() throws IOException {
        Object result = null;

        boolean hasMessage = getInput().readBoolean();
        if (hasMessage) {
            try {
                result = getInput().readObject();
            } catch (ClassNotFoundException cnfe) {
                LOG.error("Unexpected class.", cnfe);

            }
        }

        return result;
    }

    protected void writeObjectMessage(Object pMessage) throws IOException {
        boolean hasMessage = (pMessage != null);
        getOutput().writeBoolean(hasMessage);
        if (hasMessage) {
            getOutput().writeObject(pMessage);
        }
        getOutput().flush();
    }

    protected void closeByInput() throws IOException {
        getInput().close();
        getOutput().close();
        fSocket.close();
    }

    protected void closeByOutput() throws IOException {
        getOutput().close();
        getInput().close();
        fSocket.close();
    }

    protected boolean readUserConflict(String pUser) throws IOException {
        getOutput().writeUTF(pUser);
        getOutput().flush();

        return getInput().readBoolean();
    }

    protected void writeUserConflict(boolean pConflict) throws IOException {
        getOutput().writeBoolean(pConflict);
        getOutput().flush();
    }

    protected boolean readOwnedBrokerWasShutdown() throws IOException {
        return getInput().readBoolean();

    }

    protected void writeOwnedBrokerWasShutdown(boolean pShutdown) throws IOException {
        getOutput().writeBoolean(pShutdown);
        getOutput().flush();
    }

    protected int readAck() throws IOException {
        return getInput().readInt();

    }

    protected void writeAck() throws IOException {
        getOutput().writeInt(MultiUserPreempt.MSG_TYPE_PREEMPT_ACK);
        getOutput().flush();
    }


    //------------------------------------------------------------
    //- Public Interface Functions


    //------------------------------------------------------------
    //- Class Interface Functions


    //------------------------------------------------------------
    //- Inner Classes


    //------------------------------------------------------------
    //- Main


}
