/*
   Copyright 2007-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.singleapp.multiuser;

import com.notuvy.singleapp.SingleApp;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import java.io.IOException;
import java.net.BindException;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.HashMap;
import java.util.Map;

/**
 * A utility that enforces the invocation of a single application instance.
 *
 * @author  murali
 */
public abstract class MultiUserSingleAppEnforcer {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger(MultiUserSingleAppEnforcer.class);

    protected static final int MSG_TYPE_DEFER_MESSAGE = 20;
    protected static final int MSG_TYPE_PREEMPT_MESSAGE = 30;
    protected static final int MSG_TYPE_PREEMPT_ACK = 31;
    protected static final int MSG_TYPE_PING = 40;

    public static final String TEST_USER_NAME = "test.user.name";


    //------------------------------------------------------------
    //- Class Functions

    public static void defer(int pPort, SingleApp pSingleApp) {
        MultiUserSingleAppEnforcer enforcer = new MultiUserDefer(pPort, pSingleApp);
        enforcer.attempt();
    }

    public static void preempt(int pPort, SingleApp pSingleApp) {
        MultiUserSingleAppEnforcer enforcer = new MultiUserPreempt(pPort, pSingleApp);
        enforcer.attempt();
    }


    //------------------------------------------------------------
    //- Instance Variables

    private final int fPort;
    private final String fUser;  // This is not static to allow unit testing.
    private ServerSocket fServer = null;
    private final SingleApp fSingleApp;
    protected Map<String,BrokerConnection> fUserMap = new HashMap<String,BrokerConnection>();
    protected BrokerConnection fClientConnection;


    //------------------------------------------------------------
    //- Constructors

    protected MultiUserSingleAppEnforcer(int pPort, SingleApp pSingleApp) {
        fPort = pPort;
        fSingleApp = pSingleApp;
        fUser = StringUtils.defaultString(System.getProperty(TEST_USER_NAME), System.getProperty("user.name"));
    }


    //------------------------------------------------------------
    //- Accessors

    public int getPort() {
        return fPort;
    }

    public String getUser() {
        return fUser;
    }

    protected ServerSocket getServer() {
        return fServer;
    }

    protected SingleApp getSingleApp() {
        return fSingleApp;
    }


    //------------------------------------------------------------
    //- Settors

    protected void setServer(ServerSocket pServer) {
        LOG.debug("Started the SingleApp broker.");
        fServer = pServer;
    }


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    private void createNewBroker() throws IOException {
        setServer(new ServerSocket(getPort(), 1));

        fUserMap.put(getUser(), null);

        Broker.createWith(this);
    }

    // If the process containing the broker dies, then this process needs to create and own a new broker.
    // Note that a port conflict could still happen because another process could also simultaneously
    // attempt a broker creation.
    protected void reattempt() {
        // The first entity to call this will create the socket listener, and thus call "takeControl()".
        // All subsequent calls will encounter a port conflict (until the first application quits), and thus call "handlePortConflict()".
        try {
            createNewBroker();
        } catch (BindException be) {
            LOG.debug("Another process started the broker, so will connect to that.");
            handlePortConflict();
        } catch (IOException ioe) {
            LOG.error("Could not listen on [" + getPort() + "].", ioe);
        }
    }

    protected boolean alreadyHaveInstanceForUser(BrokerConnection pIncomingConnection) {
        String user = pIncomingConnection.getUser();
        BrokerConnection connection = fUserMap.get(user);
        if ((connection != null) && !connection.canCommunicate()) {
            // The connection could have died.  If we detect that it did, then invalidate that
            // connection and remove the record.
            fUserMap.remove(user);
        }
        return fUserMap.containsKey(user);
    }

    protected void handlePing() throws IOException {
        // After a ping message was received, send one back.
        fClientConnection.getOutput().writeInt(MSG_TYPE_PING);
        fClientConnection.getOutput().flush();
    }

    protected void startBrokerClientListener(BrokerConnection pBrokerConnection, Runnable pListener) {
        LOG.debug("Connecting to the broker as a client.");
        fClientConnection = pBrokerConnection;
        // Start thread to listen to requests on this socket.

        Thread thread = new Thread(pListener, "SingleAppBrokerClient");
        thread.setDaemon(true);
        thread.start();
        // The broker is running in a different instance, but we registered and can now launch.
    }


    //------------------------------------------------------------
    //- Public Interface Functions

    public void attempt() {
        // The first entity to call this will create the socket listener, and thus call "takeControl()".
        // All subsequent calls will encounter a port conflict (until the first application quits), and thus call "handlePortConflict()".
        try {
            createNewBroker();

            getSingleApp().run();
        } catch (BindException be) {
            handlePortConflict();
        } catch (IOException ioe) {
            LOG.error("Could not listen on [" + getPort() + "].", ioe);
        }
    }


    //------------------------------------------------------------
    //- Class Interface Functions

    /**
     * If a conflict with an existing broker happens, define how to proceed.
     */
    protected abstract void handlePortConflict();

    /**
     * When the first instance receives a socket connection, handle it.
     * @param pSocket  The socket with the new incoming connection.
     */
    protected abstract void handleConnection(Socket pSocket);

    /**
     * Handle the message relayed from the broker.
     * @throws IOException
     */
    protected abstract void handleRelayedMessage() throws IOException;


    //------------------------------------------------------------
    //- Inner Classes


    //------------------------------------------------------------
    //- Main



}
